class FloatList extends Object;

var FloatList next;
var byte allocated;
var float data[64];
var bool bUsed;

static function FloatList Concatenate(FloatList A, FloatList B)
{
    local int cpos;
    local FloatList cur, NA;
    
    NA = A.Duplicate();
    NA.Extend(B);
    
    return NA;
}

function Destroyed()
{
    if ( next != None )
        next.Destroy();
}

function Extend(FloatList Other)
{
    local int psource, pdest;
    local FloatList csource, cdest;
    
    if ( next != None )
        for ( cdest = next; cdest.next != None; cdest = cdest.next );
        
    else
        cdest = self;
    
    pdest = cdest.allocated;
    csource = Other;
    
    while ( psource < csource.allocated )
    {
        data[pdest] = cur.data[psource];
    
        psource++;
        pdest++;
        
        if ( psource == 64 )
        {
            if ( csource.next == None )
                return;
                
            csource = csource.next;
            psource = 0;
        }
        
        if ( pdest == 64 )
        {
            if ( cdest.next == None )
                cdest.next = New("FloatList");
                
            cdest = cdest.next;
            pdest = 0;
        }
    }
}

function FloatList Duplicate()
{
    local FloatList first, cur, cursrc;
    local int cpos;
    
    first = New("FloatList");
    cur = first;
    cursrc = self;
    
    while ( True )
    {
        for ( cpos = 0; cpos < cursrc.allocated; cpos++ )
            cur.data[cpos] = cursrc.data[cpos];
            
        cur.allocated = cursrc.allocated;
        
        cursrc = cursrc.next;
        
        if ( cursrc == None )
            break;
            
        cur.next = New("FloatList");
        cur = cur.next;
    }
    
    return first;
}

function Tick(float TimeDelta)
{
    if ( bUsed && allocated < 1 )
        Destroy();
}

function int Length()
{
    local int res;
    local FloatList fl;
    
    for ( fl = self; fl != None; fl = fl.next )
        res += fl.allocated;
        
    return res;
}

function int AddFloat(float f)
{
    local FloatList last;

    if ( allocated == 64 )
    {
        for ( last = self; last.next != None; last = last.next );
    
        if ( last == self )
        {
            next = Create(class'SectorList.FloatList');
            last = next;
        }
            
        return last.AddFloat(f);
    }
    
    data[allocated++] = f;
    return allocated;
}

function LeftShift(int pos)
{
    local FloatList sec;
    
    while ( sec = self; sec != None && position > 63; sec = sec.next ) position -= 64;
    
    if ( sec == None )
        return;
    
    while ( sec != None )
    {
        while ( pos < sec.allocated - 1 ) sec.data[pos] = sec.data[++pos];
        
        if ( sec.allocated == 64 )
        {
            if ( sec.next != None )
                sec.data[pos] = sec.next.data[0];
                
            else
            {
                sec.data[pos] = 0;
                sec.allocated--;
                
                break;
            }
                
            pos = 0;
            sec = sec.next;
        }
            
        else
        {
            data[pos] = 0; // deinit
            sec.allocated--;
            
            if ( sec.allocated == 0 )
            {
                sec.Destroy();
            }
            
            break;
        }
    }
}

function bool RemoveFloat(int position)
{
    local FloatList sect;

    if ( position > 63 )
    {
        for ( sect = self; sect != None && position > 63; sect = sect.next ) position -= 64;
        
        if ( sect == None )
            return false;
        
        return sect.RemoveFloat(position);
    }
    
    if ( position < allocated )
        return false;
        
    LeftShift(position);
    return true;
}

function OptionalFloat Get(int position)
{
    local FloatList sect;

    if ( position > 63 )
    {
        for ( sect = self; sect != None && position > 63; sect = sect.next ) position -= 64;
        
        if ( sect == None )
            return OptionalFloat.static.Empty();
        
        return OptionalFloat.static.Of(sect.data[position]);
    }
    
    return OptionalFloat.static.Of(data[position]);
}

function AddRepeated(float other, int repeats)
{
    if ( repeats < 1 )
        return;
        
    while ( repeats > 1 )
    {
        AddFloat(other);
        repeats--;
    }
    
    return AddFloat(other);
}

function Repeat(int repeats)
{
    while ( repeats > 0 )
    {
        Extend(self);
        repeats--;
    }
}